<?php
namespace App\Http\Controllers\V1;
use App\Http\Controllers\Controller;
use App\Http\Requests\V1\DestroyEventsRequest;
use App\Http\Requests\V1\EventIndexRequest;
use App\Http\Requests\V1\EventStoreRequest;
use App\Http\Resources\V1\EventCollection;
use App\Http\Resources\V1\EventResource;
use App\Models\Event;
use Carbon\Carbon;
use Illuminate\Http\JsonResponse;
use Illuminate\Http\Request;
use Illuminate\Http\Response;

class EventController extends Controller
{
    /**
     * Display a listing of the resource.`
     **/
    public function index(EventIndexRequest $request)
    {
        $searchTerm = trim($request->input('search',''));
        $app = $request->input('app', null);
        $tenant = $request->input('tenant_id', null);
        $module = $request->input('module', null);
        $page = $request->input('page',1);
        $perPage = $request->input('per_page') ?? 15;
        $sortBy = $request->input('sort_by');
        $sortDesc = $request->input('sort_desc');
        $startDate = $request->input('start_date');
        $endDate = $request->input('end_date');

        function createCarbonFromDate($date)
        {
            if (is_numeric($date)){
                return Carbon::createFromTimestamp($date);
            }else{
                return Carbon::parse($date);
            }
        }


        if(is_null($sortBy)){
            $sortBy = 'created_at';
        }

        if(is_null($sortDesc)){
            $sortDesc = 'desc';
        }


        $event = Event::orderBy($sortBy, $sortDesc)
            ->when(!is_null($app), function ($query) use ($app){
                $query->where('app', 'like', '%'.$app.'%');
            })
            ->when(!is_null($tenant), function ($query) use ($tenant){
                $query->where('tenant_id', 'like', '%'.$tenant.'%');
            })
            ->when(!is_null($module), function ($query) use ($module){
                $query->where('module', 'like', '%'.$module.'%');
            })
            ->when($startDate && $endDate, function ($query) use ($startDate, $endDate){
                $startDate = createCarbonFromDate($startDate);
                $endDate = createCarbonFromDate($endDate);
                return $query->whereBetween('created_at', [$startDate, $endDate]);
            })
            ->where(function ($query) use($searchTerm){
                $query->where('event', 'like', '%'.$searchTerm.'%')
                ->orwhere('username', 'like', '%'.$searchTerm.'%')
                ->orwhere('device_info', 'like', '%'.$searchTerm.'%')
                ->orwhere('client_ip', 'like', '%'.$searchTerm.'%');
            })
            ->paginate($perPage, ['*'], 'page', $page);


        return new EventCollection($event);
    }

    /**
     * Store a newly created resource in storage.
     * v1/api/events?search=&app=&tenant_id=&module=&page=&per_page=
     */
    public function store(EventStoreRequest $request): JsonResponse
    {
        $userAgent = $request->device_info;

        $browser = $userAgent;

       if (strpos($userAgent, 'Edg') !== false) {
            $browser = 'Microsoft Edge';
        }elseif (strpos($userAgent, 'Firefox') !== false) {
            $browser = 'Mozilla Firefox';
        }  elseif (strpos($userAgent, 'Opr') || strpos($userAgent, 'OPR') !== false) {
           $browser = 'Opera';
       }elseif (strpos($userAgent, 'Chrome') !== false) {
            $browser = 'Google Chrome';
        }elseif (strpos($userAgent, 'Safari') !== false) {
            $browser = 'Apple Safari';
        }



        $event = Event::create([
            'app' => $request->app,
            'tenant_id' => $request->tenant_id,
            'module' => $request->module,
            'event' => $request->event,
            'payload' => $request->payload,
            'username' => $request->username,
            'user_id' => $request->user_id,
            'client_ip' => $request->client_ip,
            'metadata' => $request->metadata,
            'device_info' => $browser
        ]);

        return response()->json([
            'event' => $event->id.' '.__('created')
        ]);
    }

    /**
     * Display the specified resource.
     */
    public function show(Event $event): EventResource
    {
        return new EventResource($event);
    }

    public function destroy(DestroyEventsRequest $request): JsonResponse
    {
        $events = Event::where('tenant_id', $request->tenant_id)->get();
        if ($events->isEmpty()){
            return response()->json([
                'message' => __('success')
            ], Response::HTTP_OK);
        }

        $events->each->delete();

        return response()->json([
            'message' => __('deleted_events_by_tenant_id').': '.$request->input('tenant_id')
        ]);
    }
}
